<?php

/**
 * @brief Pinba log router
 * @note following options should be enabled in config
 *     Yii::getLogger()->autoFlush = 1;
 *     Yii::getLogger()->autoDump = 1;
 * 
 * "A" - Team:
 * @author     Alexander Radionov <alex.radionov@gmail.com>
 * @author     Alexey Salov <a.salov@2gis.ru>
 */
class DGPinbaLogRoute extends CLogRoute
{

    /**
     * @brief list of pinba timers
     * @var array
     */
    private $timers = array();
    /**
     * @brief true if pinba extension exists
     * @var boolean
     */
    private $pinbaEnabled;
    /**
     * @brief true if you want send current time tag 
     * @var boolean
     */
    public $sendTimeTag = false;
    
    /**
     * @brief true if you want to combine timers with similar names
     * @var boolean
     */
    public $enableContinuation = false;

    /**
     * @brief init log router
     */
    public function init()
    {
        parent::init();
        $this->pinbaEnabled = function_exists('pinba_timer_start');
    }

    /**
     * Stores log messages into pinba.
     * @param array $logs list of log messages
     */
    protected function processLogs($logs)
    {
        // check the pinba
        if ($this->pinbaEnabled) {
            // work only with first element
            $_log = $logs[0];
            // profile log message have begin: and end: prefixes
            if ($_log[1] == 'profile') {
                if ($_log[0][0] == 'b') {
                    // begin
                    $message = substr($_log[0], 6, strlen($_log[0]));
                    $data = array(
                        'message' => $message,
                        'category' => $_log[2],
                    );
                    if ($this->sendTimeTag) {
                        $data['time'] = time();
                    }
                    $this->startTimer($message, $data);
                } else {
                    //end
                    $message = substr($_log[0], 4, strlen($_log[0]));
                    $this->stopTimer($message);
                }
            } else {
                // other levels such as trace and etc
                $this->stopTimer($_log[1]);
                $data = array(
                    'message' => $_log[0],
                    'category' => $_log[2]
                );
                if ($this->sendTimeTag) {
                    $data['time'] = time();
                }
                $this->startTimer($_log[1], $data);
            }
        }
    }

    /**
     * @brief Start named pinba timer with data
     * @param string $name
     * @param array $data
     */
    private function startTimer($name, $data)
    {
        if (isset($this->timers[$name]) && !$this->enableContinuation) {
            $number = count($this->timers[$name]) + 1;
            $data['message'] = $data['message'] . '[' . (string)$number . ']';
        } else {
            $number = 1;
        }
        $this->timers[$name][$number]['id'] = pinba_timer_start($data);
        $this->timers[$name][$number]['enabled'] = true;
    }

    /**
     * @brief Stop named pinba timer if it exists
     * @param string $name 
     */
    private function stopTimer($name)
    {
        if (isset($this->timers[$name])) {
            $timer = $this->timers[$name][count($this->timers[$name])];
            if ($timer['enabled']) {
                pinba_timer_stop($timer['id']);
                $this->timers[$name][count($this->timers[$name])]['enabled'] = false;
            } else {
                if (YII_DEBUG) {
                    throw new Exception("Timer ---{$name}--- already stopped!");
                }
            }
        }
    }

    /**
     * @brief destructor. stops all timers
     */
    public function __destruct()
    {
        if ($this->pinbaEnabled) {
            pinba_timers_stop();
        }
    }

}
